/**
 * apeman react package for image component.
 * @constructor ApImage
 */

"use strict";

const React = require('react'),
    ReactDOM = require('react-dom'),
    classnames = require('classnames'),
    extend = require('extend'),
    numcal = require('numcal'),
    _scaledSize = require('./_scaled_size'),
    ApSpinner = require('apeman-react-spinner')['ApSpinner'],
    types = React.PropTypes;

/** @lends ApImage */
let ApImage = React.createClass({


    //--------------------
    // Specs
    //--------------------

    propTypes: {
        scale: types.oneOf([
            'fit',
            'fill',
            'none'
        ]),
        width: types.number,
        height: types.number,
        src: types.string,
        alt: types.string,
        spinnerTheme: types.string
    },

    mixins: [],

    statics: {
        scaledSize: _scaledSize,
        zeroIfNaN: function (value) {
            return isNaN(value) ? 0 : value;
        }
    },

    getInitialState: function () {
        let s = this;
        return {
            imgWidth: "auto",
            imgHeight: "auto",
            loading: !!s.props.src,
            error: null
        };
    },

    getDefaultProps: function () {
        return {
            scale: 'none',
            width: null,
            height: null,
            src: null,
            alt: "NO IMAGE",
            spinnerTheme: 'a'
        };
    },

    render: function () {
        let s = this,
            state = s.state,
            props = s.props;

        let size = {
            width: props.width || 'auto',
            height: props.height || 'auto'
        };

        return (
            <div className={classnames('ap-image', props.className, {
                'ap-image-loading': props.src && state.loading
            })}
                 style={extend({}, size, props.style)}>
                {state.error ? s._renderNotfound(size) : s._renderImg(size)}
                {state.loading ? s._renderSpinner(size): ''}
            </div>
        );
    },


    //--------------------
    // Lifecycle
    //--------------------

    componentWillMount: function () {
        let s = this;
    },

    componentDidMount: function () {
        let s = this;
    },

    componentWillReceiveProps: function (nextProps) {
        let s = this;

        let src = s.props.src,
            nextSrc = nextProps.src;
        s.setState({
            loading: !!nextSrc && (nextSrc !== src)
        });
    },

    shouldComponentUpdate: function (nextProps, nextState) {
        let s = this;
        return true;
    },

    componentWillUpdate: function (nextProps, nextState) {
        let s = this;
    },

    componentDidUpdate: function (prevProps, prevState) {
        let s = this;
    },

    componentWillUnmount: function () {
        let s = this;
    },

    //------------------
    // Helper
    //------------------

    onLoad: function (e) {
        let s = this,
            props = s.props;

        let elm = ReactDOM.findDOMNode(s);

        let frameSize = {
                width: elm.offsetWidth,
                height: elm.offsetHeight
            },
            contentSize = {
                height: e.target.height,
                width: e.target.width
            },
            scaledSize = ApImage.scaledSize(
                contentSize, frameSize, props.scale
            );

        s.setState({
            imgWidth: scaledSize.width,
            imgHeight: scaledSize.height,
            loading: false
        });
    },

    onError: function (e) {
        let s = this;
        s.setState({
            error: e,
            loading: false
        });
    },

    //------------------
    // Private
    //------------------
    _renderImg: function (size) {
        let s = this,
            state = s.state,
            props = s.props;

        let zeroIfNaN = ApImage.zeroIfNaN;

        return (
            <img src={props.src}
                 alt={props.alt}
                 className={classnames('ap-image-content')}
                 style={{
                        top:zeroIfNaN((size.height - state.imgHeight) / 2),
                        left:zeroIfNaN((size.width - state.imgWidth) / 2),
                        width:state.imgWidth,
                        height:state.imgHeight
                     }}
                 onLoad={s.onLoad}
                 onError={s.onError}/>
        );
    },
    _renderNotfound: function (size) {
        let s = this,
            props = s.props;

        return (
            <div className="ap-image-notfound"
                 style={{
                    lineHeight:`${size.height}px`,
                    fontSize:`${numcal.min(size.height * 0.5, 24)}`
                 }}
            >{props.alt}</div>
        );
    },
    _renderSpinner: function (size) {
        let s = this,
            props = s.props;
        return (
            <ApSpinner className="ap-image-spinner"
                       theme={props.spinnerTheme}
                       style={{
                       width: size.width,
                       height:size.height
                       }}/>
        );
    }
});

module.exports = ApImage;